--
-- firewood - fai la legna con la motosega pulendo dai rami i tronchi tagliati, tagliando punte ecc. (taglia! taglia!! taglia!!!)
--
-- by @fcelsa
-- 
-- description for version 1.2.0 update (2016/02) 
-- when you use any type of chainsaw this script add an hud and activate a kind of mission for which it's possible to collect
-- firewood; when the virtual pallet of firewood is full (about 1000kg) pallet number are increment by one, and when you want
-- you can sell all pallet collected, and some woodchip are added to farm silo.
-- The price of firewood are fixed hardcoded to 360, but may vary, depending from type and volume of wood, temperature of the day,
-- difficult level, and some random great request.
-- All economy and quantity are limited to mission in progress, all firewood not sold at exit, will be lost! because I don't want to register new fruit_type.
-- 
-- I can't spawn any kind of pallet at this time, and wood harvester not involved in this process.
-- have fun with firewood mod
--
-- v. 1.2.0 changes
--    - the script reduce price of all trees according to game difficulty setting, for more realistic behaviour
--    - the measure can be reset with left_shift + click mouse right.
--    - slice of wood from 0,1 to 1,99 meter can be wiped out with Q key and equivalent weight added to pallet of firewood
--    - collected pallet can be sold with R key.
--    - species of tree, length, volume and remaining number of branches can be read on upper, and you can see planted pine growth state.
--    - some cosmetic changes
--    - multiplayer fixed
--    - hud problem on some resolution fixed
--
-- v. 1.2.1 fix warning in multiplayer support
--          fix for controllers and Joystick (I hope)
--    
-- thanks to all modder for great work, that they have been a source of inspiration for their clean code, and particularly:
--  V!rus(Lukas Reiche), measure mod inspiration and some tips on translations into German and encuragement
--  Jakob Tischler, for clear commented script, source of ideas and... ready to use code ;)
--  DeckerMMIV, mor2000, marhu and others, I do not remember all the code that I have studied!


local function round(n, precision)
	if precision and precision > 0 then
		return math.floor((n * math.pow(10, precision)) + 0.5) / math.pow(10, precision);
	end;
	return math.floor(n + 0.5);
end;

local targetAspectRatio = 16/9; -- = 1920/1080;
local aspectRatioRatio = g_screenAspectRatio / targetAspectRatio;
local sizeRatio = 1;
if g_screenWidth > 1920 then
	sizeRatio = 1920 / g_screenWidth;
elseif g_screenWidth < 1920 then
	sizeRatio = math.max((1920 / g_screenWidth) * .75, 1);
end;

local function getFullPx(n, dimension)
	if dimension == 'x' then
		return round(n * g_screenWidth) / g_screenWidth;
	else
		return round(n * g_screenHeight) / g_screenHeight;
	end;
end;

-- px are in targetSize for 1920x1080
local function pxToNormal(px, dimension, fullPixel)
	local ret;
	if dimension == 'x' then
		ret = (px / 1920) * sizeRatio;
	else
		ret = (px / 1080) * sizeRatio * aspectRatioRatio;
	end;
	if fullPixel == nil or fullPixel then
		ret = getFullPx(ret, dimension);
	end;
	return ret;
end;


firewood = {};

local modItem = ModsUtil.findModItemByModName(g_currentModName);
firewood.version = (modItem and modItem.version) and modItem.version or "?.?.?";

firewood.segaon = nil;
firewood.cutOn = false;
firewood.cutSt = false;
firewood.cutHz = false;
firewood.cutDl = false;
firewood.misura = 0;
firewood.nomPezzi = "";
firewood.txtPezzi = "";
firewood.lunPezzi = 0;
firewood.diaPezzi = 0;
firewood.masPezzi = 0;
firewood.volPezzi = 0;
firewood.fontSize = 0.014;
firewood.statString = " ";
firewood.econStringA = " ";
firewood.econStringB = " ";
firewood.barString = "";
firewood.cutCNT = 0;
firewood.numCNT = 1;
firewood.pagLegna = 0;
firewood.dir = g_currentModDirectory;
firewood.customerRequest = 1;
firewood.flagGDMsg = false;
firewood.chpMulti = 1;
firewood.volMulti = 1;
firewood.pineGrowing = false;
firewood.secTimer = 0;

-- Register as event listener
addModEventListener(firewood);

-----------------------------------------------------------------------------
-- funzioni specifiche
-----------------------------------------------------------------------------

function firewood.resetSetRendOpt()
	-- reset settaggi testo a beneficio delle altre mod
	setTextAlignment(RenderText.ALIGN_LEFT);
	setTextColor(1, 1, 1, 1);
	setTextBold(false);
end;

function firewood.getFirewoodPrice()
	-- il prezzo base della legna è hardcoded
	local przLegna = math.floor(360 / g_currentMission.missionStats.difficulty);
	-- aumenta il prezzo se la temperatura media della giornata è 14 o meno gradi
	local temperature = math.floor((g_currentMission.environment.weatherTemperaturesDay[1] + g_currentMission.environment.weatherTemperaturesNight[1]) / 2 );
	if temperature <= 14 then 
		przLegna = math.floor(przLegna + (przLegna * (0.30/g_currentMission.missionStats.difficulty))); 
	end;
	return przLegna * firewood.customerRequest;
end;

function firewood:addResMoney(addmoney, addchip)
	if g_currentMission:getIsServer() then
		playSample(g_currentMission.cashRegistrySound, 1, 1, 0);
		g_currentMission:addSharedMoney(addmoney, "other");
		g_currentMission:addMoneyChange(addmoney, FSBaseMission.MONEY_TYPE_SINGLE, true);
		--if g_currentMission.missionDynamicInfo.isClient == nil then  -- i woodchip vengono aggiunti solo in SP ??
		if g_server ~= nil then  -- se i silo possono essere aggiornati solo lato server, questo dovrebbe funzionare...
			g_currentMission:setSiloAmount(Fillable.FILLTYPE_WOODCHIPS, g_currentMission:getSiloAmount(Fillable.FILLTYPE_WOODCHIPS) + addchip );
		end;
	else
		g_client:getServerConnection():sendEvent(firewoodEvent:new(addmoney, addchip));
	end;
end

function firewood:deleteSlice(shape)
	if g_currentMission:getIsServer() and shape ~= nil and shape ~= 0 then
		delete(shape);
	else
		g_client:getServerConnection():sendEvent(firewoodShapeEvent:new(shape));
	end;
end

-----------------------------------------------------------------------------
-- funzioni di base game
-----------------------------------------------------------------------------

function firewood:loadMap(name)
	g_currentMission.firewoodBase = self;

	-- riduce il valore degli alberi in funzione del livello di difficoltà di gioco - commentare questo blocco di 3 righe sotto se non si vuole questo.
	for i=1, #SplitUtil.splitTypes do
		SplitUtil.splitTypes[i].pricePerLiter = SplitUtil.splitTypes[i].pricePerLiter / ( g_currentMission.missionStats.difficulty * 1.2);
	end;

	
	if (firewood.segaon == nil) then
		firewood.segaon = false;
	end;

	self.woodOverlay = createImageOverlay(Utils.getFilename("legna.dds",firewood.dir));
	self.woodOverlayTime = 0;
	self.bgOverlay = createImageOverlay("dataS2/menu/blank.png");
	setOverlayColor(self.bgOverlay, 0,0,0,0.6);
	self.graphOverlay = createImageOverlay("dataS/scripts/shared/graph_pixel.png");
	self.fweffectSample = createSample("fweffectSample");
	loadSample(self.fweffectSample,Utils.getFilename("fweffect.wav",firewood.dir), false);
	
	-- random grande richiesta 1 su 9 per un valore compreso fra 1.1 e 2.0 
	local gtRnd = math.random(10);
	if gtRnd > 9 then
		self.customerRequest = round(math.random(), 1) + 1; 
		self.flagGDMsg = true;
	end;

end;

function firewood:deleteMap()
end;

function firewood:mouseEvent(posX, posY, isDown, isUp, button)

	if isDown and button == 3 then
		if Input.isKeyPressed(Input.KEY_lshift) and firewood.segaon then
			local p = nil;
            if g_currentMission.player ~= nil then
                p = g_currentMission.player.rootNode;
            end;
			firewood.misura = 0;
			aX1, aY1, aZ1 = getWorldTranslation(p);
			g_currentMission:showBlinkingWarning(g_i18n:getText("FW_ZEROMT"), 3000);
		end;
	end;

end;

function firewood:keyEvent(unicode, sym, modifier, isDown)
end;

function firewood:update(dt)
	--if g_client == nil then return; end;
	if g_currentMission.paused then return; end;
	self.secTimer = self.secTimer + dt*0.001;
	-- attivo e disattivo il controllo del taglio della legna
	if InputBinding.isPressed(InputBinding.ACTIVATE_HANDTOOL) then
		if (not firewood.segaon) then
            local p = nil;
            if g_currentMission.player ~= nil then
                p = g_currentMission.player.rootNode;
            end;
            if p ~= nil then
				if g_currentMission.player.usesChainsaw then
					firewood.segaon = true;
					aX1, aY1, aZ1 = getWorldTranslation(p);
				else
					firewood.segaon = false;
				end;
            else
                print("firewood mod in esecuzione... player non ancora disponibile");
            end;
        end;
	end;
	
    if not(g_currentMission.player.usesChainsaw) then
		firewood.segaon = false;
		firewood.misura = 0;
		firewood.csHe = 1;
		firewood.ovhCount = 0;
		return;
	end;
	
	if firewood.flagGDMsg then
		g_currentMission.inGameMessage:showMessage(g_i18n:getText('FW_REQTITLE'), g_i18n:getText('FW_REQMESSAGE') .. "  " .. tostring(firewood.customerRequest), 0, false);
		firewood.flagGDMsg = false;
	end;
	
	local addmoney = 0;
	local addchip = 0;
	
	pagLegna = firewood.getFirewoodPrice()
	firewood.econStringA = g_i18n:getText('FW_PREZZO') .. (' %s '):format(g_i18n:formatMoney(pagLegna)) .. " / t.";
	if firewood.customerRequest ~= 1 then
		firewood.econStringA = firewood.econStringA .. " (x" .. tostring(firewood.customerRequest) .. ") ";
	end;
	
	local p = nil;	
    if g_currentMission.player ~= nil then
		p = g_currentMission.player.rootNode;
    end;
    if p ~= nil and firewood.segaon then		
		firewood.cutOn = g_currentMission.player.currentTool.isCutting;
		firewood.cutSt = g_currentMission.player.currentTool.waitingForResetAfterCut;
		firewood.cutHz = g_currentMission.player.currentTool.isHorizontalCut;
		firewood.cutDl = g_currentMission.player.currentTool.particleSystems.isEmitting;				
		-- coordinate player e calcolo metri.
		local bX, bY, bZ = getWorldTranslation(p);
		local c2 = ( math.pow( (aX1 - bX), 2 ) + math.pow( (aZ1 - bZ), 2 ) );
		firewood.misura = math.sqrt( ( c2 + math.abs( (aY1 - bY), 2 ) ) );
		firewood.misura = math.floor(firewood.misura * 100 + 0.5) / 100;
		firewood.statString = g_i18n:getText('FW_LEGNA') .. " Kg " .. tostring(firewood.cutCNT);
		firewood.barString = tostring(math.floor((firewood.cutCNT/1000)*100)) .. "%";
		-- informazioni sui pezzi 
		local shape = 0;
		local pSpl = g_currentMission.player.currentTool.cutNode;
		local x,y,z = getWorldTranslation(pSpl);
		local nx,ny,nz = localDirectionToWorld(pSpl, 1,0,0);
		local yx,yy,yz = localDirectionToWorld(pSpl, 0,1,0);	
		local minY,maxY, minZ,maxZ;
		shape, minY,maxY, minZ,maxZ = findSplitShape(x,y,z, nx,ny,nz, yx,yy,yz, 1.1, 1);
		if shape ~= 0 and shape ~= nil then
			local shx,shy,shz = getWorldTranslation(shape);
			local sx,sy,_ = project(shx,shy,shz);
			splitType = SplitUtil.splitTypes[getSplitType(shape)];
			self.txtPezzi = "";
			self.volMulti = getVolume(shape);
			self.chpMulti = round(splitType.woodChipsPerLiter, 5); -- per ora non ci serve 
			self.nomPezzi = splitType.name;
			self.masPezzi = getMass(shape);	--getMass funziona solo lato server!		
			local sizeX, sizeY, sizeZ, numConvexes, numAttachments = getSplitShapeStats(shape);
			local bdyPezzi = getRigidBodyType(shape)  -- se è Static è un albero in piedi.
			if sizeX ~= nil and self.volMulti > 0 then
				local bvVolume = sizeX*sizeY*sizeZ;
				local maxSize = math.max(sizeX, math.max(sizeY, sizeZ));
				self.txtPezzi = "  m. " .. tostring(round(maxSize,1)) .. "  vol." .. tostring(round(bvVolume,3)) .. "  att." .. tostring(numAttachments);
				if g_currentMission.showHelpText and maxSize < 2.0 and maxSize > 0.1 and self.masPezzi ~= 1 then
					g_currentMission:addHelpButtonText(g_i18n:getText("FW_QSLICE"), InputBinding.FW_QSLICE);
				end;
				
				if Input.isKeyPressed(Input.KEY_q) then
					--debug
					-- print(self.nomPezzi .. " " .. firewood.txtPezzi .. " volM:" .. tostring(round(self.volMulti,3)) .. " masPz:" .. tostring(self.masPezzi) .. "  body: " .. getRigidBodyType(shape));
					
					if Input.isKeyPressed(Input.KEY_lshift) and bdyPezzi == "Static" then
						bdyPezzi = " "; -- trucco per mettere via anche i ceppi che normalmente non voglio far sparire... salvo che il ceppo sia in terra per oltre 2 metri...
					end;

					if maxSize < 2.0 and maxSize > 0.1 and bdyPezzi ~= "Static" then 
						self.cutCNT = self.cutCNT + math.floor(self.volMulti*1000);
						self:deleteSlice(shape);
						playSample(self.fweffectSample, 1, 1, 0);
					end;
					if maxSize > 3 and maxSize < 6 and bvVolume > 0.5 then 
						--local debugMessage = firewood.setTraslationInfo(p, shape);
						g_currentMission:showBlinkingWarning("not suitable for firewood - cut again or sell to sawmill", 4000);
					end;
				end;
				-- è un pino piantato che sta crescendo ? se è meno di 2.2 probabilmente è un ceppo.
				if firewood.volMulti < 3 and splitType.splitType == 2 and firewood.cutHz and maxSize > 2.2 and self.masPezzi == 1 and bdyPezzi == "Static" then 
					firewood.pineGrowing = true;
				else
					firewood.pineGrowing = false;
				end;
			end;
		else
			self.txtPezzi = "";
		end;
		----------------------
		
		-- tagli verticali...
		if firewood.cutDl and not(firewood.cutHz) then
			if math.random(10) > 7 then
				firewood.cutCNT = firewood.cutCNT + 1;
			end;
		end;
		-- sto defogliando...
		if not(firewood.cutOn) and firewood.cutDl then
			if math.random(10) > 6 then
				firewood.cutCNT = firewood.cutCNT + 1;
			end;
		end;
		-- se i tagli sono orizzontali, es. abbattimento del tronco, non attivo il metro
		if firewood.cutHz then 
			firewood.misura = 0;
		end;
		-- azzero la misura dopo la fine di un taglio e reimposto la posizione
		if firewood.cutSt then 
			firewood.misura = 0;
			aX1, aY1, aZ1 = getWorldTranslation(p);
		end;

		if self.cutCNT >= 1000 then
			local eccedenza = self.cutCNT - 1000; 
			self.numCNT = self.numCNT +1;
			self.cutCNT = eccedenza;
			eccedenza = 0;
		end;

		if Input.isKeyPressed(Input.KEY_r) and self.numCNT >= 2 then
			local quanteff = (1000*self.numCNT-1) + self.cutCNT;
			local addmoney = math.floor((pagLegna * quanteff) / 1000);
			local addchip = math.floor(quanteff / g_currentMission.missionStats.difficulty);
			self.woodOverlayTime = 8000;
			self.econStringB = g_i18n:getText('FW_PAGAM') .. (' %s '):format(g_i18n:formatMoney(addmoney)) .. "\n" .. g_i18n:getText('FW_CIPP') .. "\n  l. " .. tostring(addchip);
			self:addResMoney(addmoney, addchip);
			self.cutCNT = 0;
			self.numCNT = 1;
			quanteff = 0; addmoney = 0; addchip = 0;
			-- debug
			-- print("firewood debug: QTA " .. tostring(quanteff) .. "  money " .. tostring(addmoney) .. "  chip " .. tostring(addchip));
		end;
    end;
	
	if self.woodOverlayTime > 0 then
		self.woodOverlayTime = self.woodOverlayTime - dt;
	end;
		
	--[[ questo forse serve in futuro...
	if shape ~= nil and shape ~= 0 then
		local cutTooLow = false;
		if getVolume(shape) < 0.1 and getMass(shape) < 0.05 then cutTooLow = true; end;
		if cutTooLow then
			g_currentMission.player.walkingIsLocked = false;
			g_currentMission.player.currentTool.curSplitShape = nil;
			shape, minY,maxY, minZ,maxZ = 0, nil,nil, nil,nil;
		end;
	end;
	--]]
	if g_currentMission.showHelpText and self.numCNT >= 2 then
		g_currentMission:addHelpButtonText(g_i18n:getText("FW_SELLFW"), InputBinding.FW_SELLFW);
	end;
end;

function firewood:draw()
	local povlX = g_currentMission.weatherTimeBackgroundOverlay.x;
	local povlY = g_currentMission.weatherTimeBackgroundOverlay.y - 0.22;
	local wdth = pxToNormal(176,'x');
	local hgth = pxToNormal(176,'y');

	if g_currentMission.renderTime and g_currentMission.player.usesChainsaw then
		renderOverlay(self.bgOverlay, povlX, povlY, (wdth*2)+0.010, hgth+0.010);
		renderOverlay(self.woodOverlay, povlX, povlY, wdth, hgth);
		setTextColor(0.95, 0.95, 0.95, 1);
		renderText(povlX+0.10, povlY+0.140, firewood.fontSize, firewood.econStringA .. " ");

		if firewood.txtPezzi ~= "" then
			local growing = " ";
			if firewood.pineGrowing then 
				setTextColor(1, 0, 0, 1);
				growing = growing .. " growing";
			else
				setTextColor(0.95, 0.95, 0.95, 1);
			end;
			renderText(povlX+0.008, povlY+0.158, firewood.fontSize, firewood.nomPezzi .. growing );
			setTextColor(0.95, 0.95, 0.95, 1);
			renderText(povlX+0.07, povlY+0.158, firewood.fontSize, firewood.txtPezzi);
			firewood.resetSetRendOpt();
		end;
		
		if firewood.segaon then
			renderText(povlX+0.008, povlY+0.01, firewood.fontSize, firewood.statString);
			local packBar = Utils.clamp((firewood.cutCNT/1000),0,1);
			if packBar > 1 then packBar = 1; end;
			firewood.resetSetRendOpt();
			
			if packBar <= 0.70  then setOverlayColor(self.graphOverlay, 0, 1, 0.5, 0.5); end;
			if packBar >= 0.75  then setOverlayColor(self.graphOverlay, 0.55, 0.55, 0, 0.5); end;
			if packBar >= 0.85  then setOverlayColor(self.graphOverlay, 1, 0.55, 0, 0.5); end;
			if packBar >= 0.95 then setOverlayColor(self.graphOverlay, 1, 0, 0, 0.5); end;
			renderOverlay(self.graphOverlay, povlX+0.09, povlY+0.034, 0.009, packBar/10);
			renderText(povlX+0.09, povlY+0.02, self.fontSize, "pallet # " .. tostring(self.numCNT) );
			
		end;
		if self.woodOverlayTime > 0 then
			renderText(povlX+0.09, povlY+0.11, firewood.fontSize, firewood.econStringB);
		end;
		if firewood.misura > 0.2 then 
			setTextColor(0, 0, 0, 1);
			renderText(0.55, 0.12, 0.024, " m. " .. ("%.2f"):format(round(firewood.misura,2)));
			setTextColor(1, 0, 0, 1);
			renderText(0.55, 0.12, 0.012, " _ _ _ _ _ _ _ _ _ _ _ _ _ _ ");
			firewood.resetSetRendOpt();
			setOverlayColor(self.graphOverlay, 1, 0.93, 0, 0.4);
			renderOverlay(self.graphOverlay, 0.548, 0.115, 0.075, 0.0325);
		end;	
		firewood.resetSetRendOpt();
	end;
	
end;

-----------------------------------------------------------------------------
-- supporto multiplayer 
-----------------------------------------------------------------------------
firewoodEvent = {};
firewoodEvent_mt = Class(firewoodEvent, Event);
InitEventClass(firewoodEvent, "firewoodEvent");
function firewoodEvent:emptyNew()
    local self = Event:new(firewoodEvent_mt);
    self.className="firewoodEvent";
    return self;
end;

function firewoodEvent:new(addmoney, addchip)
    local self = firewoodEvent:emptyNew()
    self.addmoney = addmoney;
    self.addchip = addchip;
    return self;
end;

function firewoodEvent:writeStream(streamId, connection)
	-- print(tostring(g_currentMission.time).. "ms - firewoodEvent:writeStream(streamId, connection)");
	streamWriteInt32(streamId, self.addmoney);
	streamWriteInt32(streamId, self.addchip);
end;

function firewoodEvent:readStream(streamId, connection)
	-- print(tostring(g_currentMission.time).. "ms - firewoodEvent:readStream(streamId, connection)");
	self.addmoney = streamReadInt32(streamId);
	self.addchip = streamReadInt32(streamId);
	self:run(connection);
end;

function firewoodEvent:run(connection)
	if self.addmoney and self.addchip then
		g_currentMission.firewoodBase:addResMoney(self.addmoney, self.addchip);
	end;
end;
-----------------------------------------------------------------------------

firewoodShapeEvent = {};
firewoodShapeEvent_mt = Class(firewoodShapeEvent, Event);
InitEventClass(firewoodShapeEvent, "firewoodShapeEvent");
function firewoodShapeEvent:emptyNew()
    local self = Event:new(firewoodShapeEvent_mt);
    self.className="firewoodShapeEvent";
    return self;
end;

function firewoodShapeEvent:new(shape)
    local self = firewoodShapeEvent:emptyNew()
    self.shape = shape;
    return self;
end;

function firewoodShapeEvent:writeStream(streamId, connection)
	-- print(tostring(g_currentMission.time).. "ms - firewoodShapeEvent:writeStream(streamId, connection)");
	writeSplitShapeIdToStream(streamId, self.shape);
end;

function firewoodShapeEvent:readStream(streamId, connection)
	-- print(tostring(g_currentMission.time).. "ms - firewoodShapeEvent:readStream(streamId, connection)");
	self.shape = readSplitShapeIdFromStream(streamId);
	self:run(connection);
end;

function firewoodShapeEvent:run(connection)
	if self.shape ~= nil and self.shape ~= 0 then 
		g_currentMission.firewoodBase:deleteSlice(self.shape);
	end;
end;

-----------------------------------------------------------------------------

print(string.format("Script loaded: firewood.lua (v%s)", firewood.version));
